package com.ruoyi.framework.aspectj;

import com.baomidou.mybatisplus.annotation.TableName;
import com.ruoyi.common.core.domain.BaseEntity;
import com.ruoyi.common.utils.EntityUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.system.domain.SysFieldChangeLog;
import com.ruoyi.system.service.CommonService;
import com.ruoyi.system.service.SysFieldChangeLogService;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.util.Date;
import java.util.Map;
import java.util.Objects;

@Aspect
@Component
public class FieldChangeLogAspect {

	@Autowired
	private SysFieldChangeLogService fieldChangeLogService;

	@Autowired
	private CommonService commonService;

	// 拦截 update 方法
	@Pointcut("execution(* com.ruoyi.*.service.*.update*(..))")
	public void updatePointCut() {}

	@Around("updatePointCut()")
	public Object logFieldChanges(ProceedingJoinPoint point) throws Throwable {
		Object[] args = point.getArgs();
		if (args.length == 0 || args[0] == null) {
			return point.proceed(); // 跳过无效参数
		}
		Object updatedEntity = args[0];

		// 判断是否包含 @PrimaryKey 注解
		if (!EntityUtils.hasPrimaryKeyAnnotation(updatedEntity)) {
			return point.proceed(); // 如果没有主键注解，跳过日志记录
		}

		// 使用工具类获取主键值和表名
		Long recordId = EntityUtils.getPrimaryKeyValue(updatedEntity);
		String tableName = EntityUtils.getTableName(updatedEntity);

		// 动态查询原始记录
		Map<String, Object> originalRecord = commonService.findById(tableName, recordId);

		// 记录字段变更日志
		saveFieldChangeLogs(originalRecord, updatedEntity, tableName, recordId);

		return point.proceed(); // 执行原方法
	}


	private void saveFieldChangeLogs(Map<String, Object> original, Object updatedEntity, String tableName, Long recordId) {
		Long userId = SecurityUtils.getLoginUser().getUserId();
		String username = SecurityUtils.getLoginUser().getUsername();

		for (Field field : updatedEntity.getClass().getDeclaredFields()) {
			field.setAccessible(true);

			try {
				String fieldName = field.getName();

				String dbFieldName = toUnderlineCase(fieldName); // 将驼峰命名转换为下划线命名


				Object oldValue = original.get(dbFieldName); // 获取原始值
				Object newValue = field.get(updatedEntity); // 获取新值

				// 如果原值和新值相等，则跳过
				if ((newValue == null) || (oldValue != null && oldValue.equals(newValue))) {
					continue;
				}

				if (!Objects.equals(oldValue, newValue)) {
					SysFieldChangeLog log = new SysFieldChangeLog();
					log.setTableName(tableName);
					log.setRecordId(recordId);
					log.setFieldName(fieldName);
					log.setOldValue(oldValue != null ? oldValue.toString() : null);
					log.setNewValue(newValue != null ? newValue.toString() : null);
					log.setUserId(userId);
					log.setUsername(username);
					log.setOperationType("修改");
					log.setCreateTime(new Date());
					fieldChangeLogService.save(log);
				}
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * 将驼峰命名转换为下划线命名
	 *
	 * @param camelCase 驼峰格式字符串（如 contactName）
	 * @return 下划线格式字符串（如 contact_name）
	 */
	public static String toUnderlineCase(String camelCase) {
		if (camelCase == null || camelCase.isEmpty()) {
			return "";
		}
		// 使用正则表达式匹配驼峰命名的规则
		return camelCase.replaceAll("([a-z])([A-Z])", "$1_$2").toLowerCase();
	}

}

